/* 
 *  Dynamixel pro operations to change address of an XL-320
 */

#include <DynamixelPro.h>

DynamixelPro dxlEX(1);  // DXL PRO(485) bus on serial 3 USART

void setup() {
  SerialUSB.begin();               // Talk back to computer
  pinMode(BOARD_LED_PIN, OUTPUT);  //CM9.04 board LED
  dxlEX.begin(3);                  //57600 bps in DXL 2.0 table
  
  SerialUSB.print("Send any value to Start...\n");
  delay(5000);
  SerialUSB.print("Send any value to Start...\n");
  while(!SerialUSB.available())
  {
    digitalWrite(BOARD_LED_PIN, LOW);
    delay(500);
    digitalWrite(BOARD_LED_PIN, HIGH);
    delay(500);
  }
  SerialUSB.read();              //read and throw away
}

int SerUSB_ATOI(int bytes)
/* 
*  Read "bytes" number of chars from SerialUSB and
*  converts them to an int.
*/
{
  int n;
  int res = 0;
  char str[20];
  
  // clear string to nulls
  for (n=0; n<20; n++) {
    str[n] = 0;
  }
  
  for (n=0; n< bytes; n++) {
    str[n] = SerialUSB.read();
  }
  res = atoi(str);
  return(res);
}

int GetServoAddr(void)
/*
*  Get a servo address, either source or destination.
*/
{
  int n=0;
  int m;
  int servo = 0;
  
  while(n == 0) {
    SerialUSB.println("address?");
    m=0;
    while (m == 0) {
      m = SerialUSB.available();
    }
    servo = SerUSB_ATOI(m);
    if (servo < BROADCAST_ID) {
      n=1;
    }
    else {
      SerialUSB.println("Bad value");
    }
  }

  return(servo);
}

void ServoScan(void)
/*
*  Look for servos on the bus.
*/
{
  int n;
  word twoByte;
  byte res;
  
  SerialUSB.println("Scanning for servos...");
  for (n=0; n<BROADCAST_ID; n++) {
    twoByte = dxlEX.readWord(n,0);
    res = dxlEX.getResult();
    if (res == COMM_RXSUCCESS){
      SerialUSB.print(n);
      SerialUSB.print(" : ");
      SerialUSB.println(twoByte);
    }
  }
}

void loop() {
  byte sAddr, dAddr;
  byte res,bLen,oneByte;
  
  ServoScan();
  
  // Get the servo address to change.
  SerialUSB.print("Change ");
  sAddr = GetServoAddr();
  SerialUSB.print("to ");
  dAddr = GetServoAddr();
  SerialUSB.print("Source      addr = ");SerialUSB.println(sAddr);
  SerialUSB.print("Destination addr = ");SerialUSB.println(dAddr);
  
  // Change the servo address
  dxlEX.writeByte((byte)sAddr, 3, (byte)dAddr);
  res = dxlEX.getResult();
  SerialUSB.print("result= ");SerialUSB.print(res,HEX);
  if (res == COMM_RXSUCCESS) {
    SerialUSB.println(" success");
  }
  else {
    SerialUSB.println(" failure");
  }
    
  ServoScan();
  
  SerialUSB.println("Another? (y/n)");
  while ((bLen = SerialUSB.available()) == 0);
  oneByte = SerialUSB.read();
  if (oneByte != 'y') {
    SerialUSB.print("Done\n");
    while(1);  // Done, wait forever
  }
  
}

